<?php

namespace HCM\TRE\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;

class PerfilCursoController extends Crud {

    protected $dataSourceName = '';
    protected $dataSourceName_perfil = 'getPerfilCurso';
    protected $dataSourceName_item_get = 'getItemPerfilCurso';
    protected $dataSourceName_item_set = 'setItemPerfilCurso';
    protected $dataSourceName_curso = 'tre_curso';

    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO');
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters , \HCM\TRE\Services\PerfilCursoService $perfilCursoService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->PerfilCursoService = $perfilCursoService;
        parent::__construct($dataSourceManager);
    }

    public function getPerfilCurso(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName_perfil);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function getCurso_Tre(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName_curso);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    
    public function setPerfilCurso(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $this->dataSourceName = $this->dataSourceName_perfil;
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRPERFILCURSO', $row)) {
                    if ($row['NRPERFILCURSO'] === '') {
                        $row['NRPERFILCURSO'] = \HCM\Util\NovoCodigo::geraCodigo( $row['NRORG'], 'GPE_PERFILCURSO', 1, 12, true);
                    }
                } else {
                    $row['NRPERFILCURSO'] = \HCM\Util\NovoCodigo::geraCodigo( $row['NRORG'], 'GPE_PERFILCURSO', 1, 12, true);
                }
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deletePerfilCurso(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $this->dataSourceName = $this->dataSourceName_perfil;
            $row = $request->getDataSet()->getRows()[0];
            if($this->PerfilCursoService->ItemExists($row['NRORG'],$row['NRPERFILCURSO'])){
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("Existem itens relacionados a este perfil, favor deletá-los","E"));
            }else{
                parent::delete($request, $response);
            }
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }    
    }
    
    public function getItemPerfilCurso(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName_item_get);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function setItemPerfilCurso(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
        $this->dataSourceName = $this->dataSourceName_item_set;
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRITPERFILCURSO', $row)) {
                    if ($row['NRITPERFILCURSO'] === '') {
                        $row['NRITPERFILCURSO'] = \HCM\Util\NovoCodigo::geraCodigo( $row['NRORG'], 'GPE_ITPERFILCURSO', 1, 12, true);
                    }
                } else {
                    $row['NRITPERFILCURSO'] = \HCM\Util\NovoCodigo::geraCodigo( $row['NRORG'], 'GPE_ITPERFILCURSO', 1, 12, true);
                }
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteItemPerfilCurso(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $this->dataSourceName = $this->dataSourceName_item_set;
            parent::delete($request, $response);
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }    
    }
    
    
    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
} 