<?php

/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

namespace Geral\Logic\PropostaComercial;

use Geral\Util\DateUtil;
use Setforms2\Shared\Registry;
use Setforms2\Runtime\Connection;
use Indeterminado\Entities\Vencservprop;
use Windows\WinInt00201000\TabInt00201000005\Facade;

/**
 * 
 * Description of NegociarServicos
 *
 * @author phillipesilva
 */

class NegociarServicos 
{    
    public static function insertParcelas($nrproposfas, $nrverpropfas, $dtvencserv, $nrorg, $vrvencservprop)
    {
        $db = \Setforms2\Runtime\Connection::getInstance();
        $db->getConn()->beginTransaction();
        $connection = \Setforms2\Shared\Registry::getEntityManager();
        
        try{            
            $vencservprop = new \Indeterminado\Entities\Vencservprop();
            $vencservprop->setNrproposfas($nrproposfas);
            $vencservprop->setNrverpropfas($nrverpropfas);
            $vencservprop->setDtvencservprop($dtvencserv);
            $vencservprop->setVrvencservprop($vrvencservprop);
            $vencservprop->setNrorg($nrorg);
            $vencservprop->persist();
            $connection->flush();

            $db->getConn()->commit();                
        }catch(\Exception $e){
            $db->getConn()->rollback();
            throw $e;
        }
    }
    
    public static function deleteParcelas($nrproposfas, $nrverpropfas, $nrorg)
    {
        $db = \Setforms2\Runtime\Connection::getInstance();
        $db->getConn()->beginTransaction();
        $connection = \Setforms2\Shared\Registry::getEntityManager();  
        
        try{
            $vencservprop = Vencservprop::findByNrproposfasAndNrverpropfasAndNrorg($nrproposfas, $nrverpropfas, $nrorg);
            
            if(is_array($vencservprop))
            {
                foreach($vencservprop as $venc)
                {
                    $venc->delete();
                }                               
            }           
             
            $connection->flush();
            $db->getConn()->commit();
            
        }catch(\Exception $e){
            $db->getConn()->rollback();
            throw $e;
        }
    }
    
    public static function recalcularVrTotalComJuros($vrTotalServ, $dtPriParcela, $qtParcelas, $txJuros)
    {
        $vrParcelaSemJuros = $vrTotalServ / $qtParcelas;
        $txJurosPriMes     = pow(1 + $txJuros / 100, 1 / 30);
        $vrTotalParcCalc   = 0;
        $dtPriParcela      = Facade::geraData($dtPriParcela, 1);
        $dataProxParc      = $dtPriParcela;
        
        for($i = 0; $i < $qtParcelas; $i++)
        {           
            $dataProxParc      = Facade::geraData($dataProxParc, 2);
            $diferencaDias     = DateUtil::getQtdeDiasEntre($dtPriParcela, $dataProxParc);
            $vrParcelaComJuros = pow($txJurosPriMes, $diferencaDias) * $vrParcelaSemJuros;
            $vrTotalParcCalc  += $vrParcelaComJuros;
        }
        
        return $vrTotalParcCalc;
    }

    public static function recalcularParcelas($dataSetParc, $parcelas, $nrOrg, $vrTotal, $dtProposta)
    {
        $db = Connection::getInstance();
        $db->getConn()->beginTransaction();
        $manager = Registry::getEntityManager();

        try {
            $vrTotalUpdate  = 0;
            $messageRetorno = false;

            //Busca o valor total das parcelas alteradas e valida se a Data informada é anterior à data da proposta
            foreach($dataSetParc['UPDATE'] as $parcUpdate) {
                $vrTotalUpdate += $parcUpdate['VRVENCSERVPROP'];
                $dtVenc = DateUtil::getDataDeString(substr($parcUpdate['DTVENCSERVPROP'], 0, 10));
                $dtProp = DateUtil::getDataDeString(substr($dtProposta, 0, 10));

                if(DateUtil::comparaDatas($dtVenc, $dtProp)) {
                    $messageRetorno = 'DATA';
                }
            }

            if($vrTotalUpdate > $vrTotal) {
                $messageRetorno = 'VRTOTAL';
            } else if($messageRetorno != 'DATA'){
                $valorRestante = $vrTotal - $vrTotalUpdate;
                $valorParcRestante = $valorRestante / count($dataSetParc['NOUPDATE']);

                foreach($parcelas as $parcela) {
                    foreach($dataSetParc['UPDATE'] as $parcUpdate) {
                        if($parcela['NRPARCELA'] == $parcUpdate['NRPARCELA']){
                            self::updateParcela($parcUpdate, $parcela, $nrOrg);
                        }
                    }
                }

                foreach($dataSetParc['NOUPDATE'] as $parcNoUpdate) {
                    $parcNoUpdate['VRVENCSERVPROP'] = $valorParcRestante;
                    self::updateParcela($parcNoUpdate, null, $nrOrg);
                }
            }

            $manager->flush();
            $db->getConn()->commit();
        } catch (\Exception $e) {
            $db->getConn()->rollback();
            throw $e;
        }

        return $messageRetorno;
    }

    public static function updateParcela($parcUpdate, $parcOld, $nrOrg)
    {
        $parcUpdate['DTVENCSERVPROP'] = substr($parcUpdate['DTVENCSERVPROP'], 0, 10);

        if($parcOld) {
            $parcOld['DTVENCSERVPROP'] = substr($parcOld['DTVENCSERVPROP'], 0, 10);
        }

        $where = "NRPROPOSFAS  = ?
              AND NRVERPROPFAS = ?
              AND NRORG        = ?
              AND TO_CHAR(DTVENCSERVPROP, 'DD/MM/YYYY') = ?";

        //Se $parcOld tiver valor significa que estão sendo levadas em consideração as parcelas que o usuário alterou.
        if($parcOld) {
            $params = array($parcUpdate['NRPROPOSFAS'],
                $parcUpdate['NRVERPROPFAS'],
                $nrOrg,
                $parcOld['DTVENCSERVPROP']);
        } else {
            $params = array($parcUpdate['NRPROPOSFAS'],
                $parcUpdate['NRVERPROPFAS'],
                $nrOrg,
                $parcUpdate['DTVENCSERVPROP']);
        }

        $vencServ = Vencservprop::getObjectByWhere($where, $params);

        if(is_object($vencServ)) {
            $vencServ->setVrvencservprop($parcUpdate['VRVENCSERVPROP']);

            if($parcOld) {
                $vencServ->setDtvencservprop(DateUtil::getDataDeString($parcUpdate['DTVENCSERVPROP'], DateUtil::FORMATO_BRASILEIRO, true));
            }

            Registry::getEntityManager()->flush();
        }
    }
}