<?php

require_once 'base/FparesumoctblEntityBase.php';


/**
 * FparesumoctblEntity.
 *
 * @package Entity
 * @author  alessandrobartels 28/04/2011
 *
 */
class FparesumoctblEntity extends FpaResumoctblEntityBase
{


    /**
     * Persiste dados na entidade de Resumo Contabil.
     *
     * @param integer          $nrorg            Numero da Organizacao.
     * @param TkDateTime       $dt_ocorrencia    Instancia da classe TkDateTime.
     * @param FpaeventomEntity $evento           Instancia da classe FpaeventomEntity.
     * @param integer          $nr_estruturam    Numero da Estrutura hierarquica pai.
     * @param array            $totaliza_eventos Array com os eventos totalizados.
     *
     * @throws Exception Persistencia nao realizada.
     * @return boolean
     *
     * @author alessandrobartels - 28/04/2011
     */
    public function persisteResumoContabil(
        $nrorg, TkDateTime $dt_ocorrencia, FpaeventomEntity $evento, $nr_estruturam, array $totaliza_eventos
    ){
        foreach ($totaliza_eventos as $nr_evento => $valor) {
            $obj_evento = $evento->retornaEvento($nr_evento);
            // Determinar tipo de lancamento (C, D).
            try {
                $this->setNrorg($nrorg);
                $this->setNrresumoctbl(rand(1, 10000));
                $this->setDtocorrencia($dt_ocorrencia);
                $this->setNrestruturam($nr_estruturam);
                $this->setIdtipolanc($idtipolanc);
                $this->setCdcontctbl($cdcontctbl);
                $this->setCdhistpadr($cdhistpadr);
                $this->setVrlancamento($vrlancamento);
                $this->setDtinclusao(now());
                $this->setDtultatu(now());
                $this->setNrorginclusao($nrorginclusao);
                $this->setCdoperinclusao($cdoperinclusao);
                $this->setNrorgultatu($nrorgultatu);
                $this->setCdoperultatu($cdoperultatu);

                $this->persist();

                return true;

            } catch (Exception $e) {
                $msg = 'Persistencia nao realizada';
                throw new Exception($msg);
                return false;
            }
        }
    }


    /**
     * Enter description here ...
     *
     * @param integer    $nr_org        Numero da Organizacao.
     * @param TkDateTime $dt_ocorrencia Data de Ocorrencia.
     * @param integer    $nr_estrutura  Numero da Estrutura.
     *
     * @return array FparesumoctblEntity
     */
    public function retornaResumoContabilPorEstruturaMesCompetencia($nr_org, TkDateTime $dt_ocorrencia, $nr_estrutura){
        $resultado = Fparesumoctbl::retornaResumoContabilPorEstruturaMesCompetencia(
            $nr_org, $dt_ocorrencia, $nr_estrutura
        );
        return $resultado;
    }


    /**
     * Enter description here ...
     *
     * @param float  $valor_evento   Valor do evento contabil.
     * @param string $conta_contabil Codigo da conta contabil.
     * @param string $historico      Historico.
     *
     * @throws Exception $e Propaga.
     * @return boolean
     */
    public function atualizaResumoContabil($valor_evento, $conta_contabil, $historico = null){
        try {
            $valor_lancamento = $this->getVrlancamento() + $valor_evento;
            $this->setCdcontctbl($conta_contabil);
            $this->setCdhistpadr($historico);
            $this->setVrlancamento($valor_lancamento);
            $this->setDtultatu(now());
            $this->persist();
            return true;
        } catch(Exception $e){
            throw new Exception($e->getMessage());
            return false;
        }
    }


    /**
     * Enter description here ...
     *
     * @param integer    $nrorg         Numero da Organizacao.
     * @param TkDateTime $dt_ocorrencia Data de ocorrencia.
     * @param integer    $nr_estruturam Numero da Estrutura.
     * @param string     $idtipolanc    Tipo de lancamento (credito/desconto).
     * @param string     $cdcontctbl    Codigo conta contabil.
     * @param string     $cdhistpadr    Historico.
     * @param float      $vrlancamento  Valor do lancamento.
     *
     * @throws Exception $e Propaga.
     * @return boolean
     */
    public function insereResumoContabil(
        $nrorg, TkDateTime $dt_ocorrencia, $nr_estruturam, $idtipolanc, $cdcontctbl, $cdhistpadr, $vrlancamento
    ){
        try {
            $this->setNrorg($nrorg);
            $this->setDtocorrencia($dt_ocorrencia);
            $this->setNrestruturam($nr_estruturam);
            $this->setIdtipolanc($idtipolanc);
            $this->setCdcontctbl($cdcontctbl);
            $this->setCdhistpadr($cdhistpadr);
            $this->setVrlancamento($vrlancamento);

            $this->persist();
            return true;
        } catch(Exception $e){
            throw new Exception($e->getMessage());
            return false;
        }

    }


    /**
     * Enter description here ...
     *
     * @param integer    $nr_org        Numero da Organizacao.
     * @param TkDateTime $dt_ocorrencia Data de Ocorrencia.
     * @param integer    $nr_estrutura  Numero da estrutura.
     *
     * @return boolean
     */
    public function criaListaResumoContabil($nr_org, TkDateTime $dt_ocorrencia, $nr_estrutura){
        $resumo_contabil = $this->retornaResumoContabilPorEstruturaMesCompetencia(
            $nr_org, $dt_ocorrencia, $nr_estrutura
        );
        foreach ($resumo_contabil as $item_resumo) {
            $this->addItem($item_resumo, $item_resumo->getNrresumoctbl());
        }

        if (is_array($resumo_contabil) === true) {
            return true;
        } else {
            return false;
        }
    }


    /**
     * Procura no Resumo Contabil registro referente a conta parametrizada.
     *
     * @param string $conta Descricao da Conta Contabil.
     *
     * @return FparesumoctblEntity
     */
    public function procuraResumoContabilPorContaContabil($conta, $nrEstrutura, $historico, $idtipolanc){
        $itens = $this->getItems();
        foreach ($itens as $item) {
            if ( ($conta === $item->getCdcontctbl())
               && ($nrEstrutura === $item->getNrestruturam())
               && ($historico === $item->getCdhistpadr())
               && ($idtipolanc === $item->getIdtipolanc())
            ) {
                return $item;
            }
        }
    }


    /**
     * Enter description here ...
     *
     * @param integer    $nrorg         Numero da Organizacao.
     * @param TkDateTime $dt_ocorrencia Data de ocorrencia.
     * @param integer    $nr_estruturam Numero da Estrutura.
     * @param string     $idtipolanc    Tipo de lancamento (credito/desconto).
     * @param string     $conta_contabil    Codigo conta contabil.
     * @param string     $historico    Historico.
     * @param float      $valor_evento  Valor do lancamento.
     *
     * @return void
     */
    public function atualizaListaResumoContabil(
        $nrorg, TkDateTime $dt_ocorrencia, $nr_estruturam, $idtipolanc, $conta_contabil, $historico, $valor_evento
    ){

        // Procura resumo na lista, por estrutura, conta e historico ?????????????? //@todo Ajustar
        $item_resumo = $this->procuraResumoContabilPorContaContabil($conta_contabil, $nr_estruturam, $historico, $idtipolanc);

        if (is_object($item_resumo) === true) {
           // tk_warning('Atualiza');
            // Atualiza.
            $valor_lancamento = $item_resumo->getVrlancamento() + $valor_evento;
            $item_resumo->setNrorg($item_resumo->getNrorg());
            $item_resumo->setCdcontctbl($conta_contabil);
            $item_resumo->setCdhistpadr($historico);
            $item_resumo->setVrlancamento($valor_lancamento);
        } else {
           // tk_warning('Insere');
            // Inclui.
            $item_resumo = new self();
            $item_resumo->setNrorg($nrorg);
            $item_resumo->setDtocorrencia($dt_ocorrencia);
            $item_resumo->setNrestruturam($nr_estruturam);
            $item_resumo->setIdtipolanc($idtipolanc);
            $item_resumo->setCdcontctbl($conta_contabil);
            $item_resumo->setCdhistpadr($historico);
            $item_resumo->setVrlancamento($valor_evento);
            
            $this->addItem($item_resumo, $item_resumo->getNrresumoctbl());
        }


    }


    /**
     * Enter description here ...
     *
     * @return void
     */
    public function retrocedeListaResumoContabil(){
        $itens = $this->getItems();
        foreach ($itens as $item) {
            // Teste.
            $this->removeItem($item);
        }
    }


}